interface DatabaseConnection<ModelType> {
	[modelName: string]: ModelType
}

interface DatabaseDriverStatic<DriverType, ModelType> {
	/** Returns database connection by its name */
	connect (connection?: string): DatabaseConnection<ModelType> & DriverType
}

export abstract class DatabaseDriver<ModelType> {
	public _name: string;
	public abstract constructor (options: object);
	public abstract connect (): Promise<void>;
	public abstract getModel (basePath: string, name: string): ModelType;
}

type Class<T> = new (...args: any[]) => T;
type Instance<ClassType> = ClassType extends Class<infer T> ? T : never;

type ExtractModelType<DriverClassType> = Instance<DriverClassType> extends DatabaseDriver<infer ModelType> ? ModelType : never;
export function registerDriver<DriverClassType> (driver: DriverClassType, driverName: string): DatabaseDriverStatic<Instance<DriverClassType>, ExtractModelType<DriverClassType>> & DriverClassType;
